#donotrun
// This is a simple shader for rendering images
// from an accumulated buffer.

#vertex

varying vec2 coord;

void main(void)
{
	gl_Position =  gl_Vertex;
	coord = (gl_ProjectionMatrix*gl_Vertex).xy;
}

#endvertex


varying vec2 coord;
uniform vec2 pixelSize;
uniform sampler2D frontbuffer;

uniform float FocalDepth; 
uniform float DepthMultiplier;  //determines the slope of DOF
uniform float DOFSize; //use this to set the amount of DOF (exponential slowdown)
vec3 BokehLumCoeff = vec3(0.299,0.587,0.114);//what colors cause highlights
uniform float BokehThreshold;//what luminance causes highlights
uniform float BokehGain; //how much should they "pop"
vec2 vDofBox=vec2(0.5*DOFSize);//half size of max search box

float GetShape(vec2 p){//for shaping bokeh highlights
	return length(p);//circle
	//return max(abs(p.x-p.y),abs(p.x+p.y));//diamond
	//return max(min(abs(p.x),abs(p.y))*6.0,length(p));//cross hair
	//return max(min(abs(p.x-p.y),abs(p.x+p.y))*6.0,length(p));//x
	//return min(max(abs(p.x),abs(p.y))*1.5,max(abs(p.x+p.y),abs(p.x-p.y)));//octo star
	//return max(max(abs(p.x),abs(p.y))*1.5,max(abs(p.x+p.y),abs(p.x-p.y)));//octogon
}
vec4 DespeckledSamp(vec2 p){//you may need to despeckle to get stable highlights
	const vec3 v=vec3(0.0,1.0,-1.0);//use these lines if no mipmapping
	vec3 cMix=(texture2D(frontbuffer,p+v.xz*pixelSize).rgb+texture2D(frontbuffer,p+v.zx*pixelSize).rgb+
		texture2D(frontbuffer,p+v.yx*pixelSize).rgb+texture2D(frontbuffer,p+v.xy*pixelSize).rgb)*0.25;
	//vec3 cMix=texture2DLod(frontbuffer,p,3.0).rgb;//alternative if mipmapped
	vec4 col=texture2D(frontbuffer,p);
	return vec4(mix(col.xyz,cMix,abs(col.xyz-cMix)),col.a);//col is forced to be like neighbors if outlier
}
void main() {
	vec2 co=(coord+vec2(1.0))/2.0;//gl_FragCoord.xy;
	vec4 dP=DespeckledSamp(co);//texture2D(frontbuffer,co);//original pixel color and depth
	//gl_FragColor = vec4(dP.rgb,1.0);
	//gl_FragDepth = clamp(dP.a,0.0,0.9);
	//return;
	vec4 color=vec4(0.0);//accumulator, using ".a" for weights
	vec2 mn=max(vec2(0.5)*pixelSize,co-vDofBox);//find the dims of seach rectangle
	vec2 mx=min(vec2(1.0),co+vDofBox);//don't wrap around
	for(float y=mn.y;y<mx.y;y+=pixelSize.y){
		for(float x=mn.x;x<mx.x;x+=pixelSize.x){
			float fLen=GetShape(co-vec2(x,y));//this can be any shape including fractals
			if(fLen>vDofBox.x)continue;//skip if not within shape boundary
			vec4 dS=DespeckledSamp(vec2(x,y));//texture2D(frontbuffer,vec2(x,y));//get the sample to blend
			float cR=clamp(DepthMultiplier*abs(dS.a-FocalDepth),0.04,1.0);//circle of confusion radius
			//next line is for bokeh highlights so comment out if no want sparkles :)
			dS.rgb+=dS.rgb*pow(cR,4.0)*BokehGain*max((dot(dS.rgb, BokehLumCoeff)-BokehThreshold), 0.0);
			color+=max(0.0,(DOFSize*cR-fLen)/(3.14159*cR*cR))*vec4(dS.rgb,1.0);//accumulate weighted samples
		}
	}//normalize the accumulated color and mix with original (put back in-focus pixels that were obliterated during blend)
	color.rgb=mix(dP.rgb,color.rgb/color.a,min(0.5*DepthMultiplier*abs(dP.a-FocalDepth),1.0));
	gl_FragColor = vec4(color.rgb,1.0);
}
